"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function cors(options) {
    const defaults = {
        allowMethods: 'GET,HEAD,PUT,POST,DELETE,PATCH'
    };
    options = options || {};
    options.allowMethods = defaults.allowMethods;
    if (Array.isArray(options.exposeHeaders)) {
        options.exposeHeaders = options.exposeHeaders.join(',');
    }
    if (Array.isArray(options.allowMethods)) {
        options.allowMethods = options.allowMethods.join(',');
    }
    if (Array.isArray(options.allowHeaders)) {
        options.allowHeaders = options.allowHeaders.join(',');
    }
    if (options.maxAge) {
        options.maxAge = String(options.maxAge);
    }
    options.credentials = !!options.credentials;
    options.keepHeadersOnError = options.keepHeadersOnError === undefined || !!options.keepHeadersOnError;
    return async (ctx, next) => {
        let requestOrigin = ctx.headers['origin'];
        if (!requestOrigin)
            return await next();
        let origin;
        if (typeof options.origin === 'function') {
            origin = await options.origin(ctx);
            if (!origin)
                return await next();
        }
        else {
            origin = options.origin || requestOrigin;
        }
        let headers = {};
        if (ctx.req.method !== 'OPTIONS') {
            headers['Access-Control-Allow-Origin'] = origin;
            if (options.credentials === true) {
                if (origin === '*') {
                    delete ctx.req.headers['Access-Control-Allow-Credentials'];
                }
                else {
                    headers['Access-Control-Allow-Credentials'] = 'true';
                }
            }
            if (options.exposeHeaders)
                headers['Access-Control-Expose-Headers'] = options.exposeHeaders;
            if (!options.keepHeadersOnError)
                return await next();
            ctx.set(headers);
            await next();
        }
        else {
            if (!ctx.headers['access-control-request-method'])
                return await next();
            headers['Access-Control-Allow-Origin'] = origin;
            if (options.credentials === true)
                headers['Access-Control-Allow-Credentials'] = 'true';
            if (options.maxAge)
                headers['Access-Control-Max-Age'] = options.maxAge;
            if (options.allowMethods)
                headers['Access-Control-Allow-Methods'] = options.allowMethods;
            let allowHeaders = options.allowHeaders;
            if (!allowHeaders)
                allowHeaders = ctx.headers['access-control-request-headers'];
            if (allowHeaders)
                headers['Access-Control-Allow-Headers'] = allowHeaders;
            ctx.set(headers);
            ctx.routed = true;
            ctx.status = 204;
            requestOrigin = headers = origin = allowHeaders = null;
        }
    };
}
exports.cors = cors;
